@@ -11,8 +11,11 @@ use File::Spec ();
 use File::Path ();
 use File::Basename ();
 
+use Module::Build::Base;
+
 use vars qw($VERSION @ISA);
-$VERSION = '0.21_02';
+@ISA = qw(Module::Build::Base);
+$VERSION = '0.24_01';
 
 # Okay, this is the brute-force method of finding out what kind of
 # platform we're on.  I don't know of a systematic way.  These values
@@ -59,25 +62,30 @@ my %OSTYPES = qw(
 		 mpeix     MPEiX
 		);
 
-# We only use this once - don't waste a symbol table entry on it.
-# More importantly, don't make it an inheritable method.
-my $load = sub {
-  my $mod = shift;
-  #warn "Using $mod";
+sub _interpose_module {
+  my ($self, $mod) = @_;
   eval "use $mod";
   die $@ if $@;
+
+  no strict 'refs';
+  my $top_class = $mod;
+  while (@{"${top_class}::ISA"}) {
+    last if ${"${top_class}::ISA"}[0] eq $ISA[0];
+    $top_class = ${"${top_class}::ISA"}[0];
+  }
+
+  @{"${top_class}::ISA"} = @ISA;
   @ISA = ($mod);
-};
+}
 
 if (grep {-e File::Spec->catfile($_, qw(Module Build Platform), $^O) . '.pm'} @INC) {
-  $load->("Module::Build::Platform::$^O");
+  __PACKAGE__->_interpose_module("Module::Build::Platform::$^O");
 
 } elsif (exists $OSTYPES{$^O}) {
-  $load->("Module::Build::Platform::$OSTYPES{$^O}");
+  __PACKAGE__->_interpose_module("Module::Build::Platform::$OSTYPES{$^O}");
 
 } else {
   warn "Unknown OS type '$^O' - using default settings\n";
-  $load->("Module::Build::Platform::Default");
 }
 
 sub os_type { $OSTYPES{$^O} }
@@ -120,7 +128,7 @@ platforms like MacOS (traditional) can use it fairly easily.  Its only
 prerequisites are modules that are included with perl 5.6.0, and it
 works fine on perl 5.005 if you can install a few additional modules.
 
-See L<MOTIVATIONS> for more comparisons between C<ExtUtils::MakeMaker>
+See L<"MOTIVATIONS"> for more comparisons between C<ExtUtils::MakeMaker>
 and C<Module::Build>.
 
 To install C<Module::Build>, and any other module that uses
@@ -135,17 +143,18 @@ This illustrates initial configuration and the running of three
 'actions'.  In this case the actions run are 'build' (the default
 action), 'test', and 'install'.  Actions defined so far include:
 
-  build                          docs        
-  clean                          fakeinstall 
-  code                           help        
-  diff                           install     
-  dist                           manifest    
-  distcheck                      ppd         
-  distclean                      realclean   
-  distdir                        skipcheck   
-  distmeta                       test        
-  distsign                       testdb      
+  build                          fakeinstall   
+  clean                          help          
+  code                           install       
+  diff                           manifest      
+  dist                           ppd           
+  distcheck                      ppmdist       
+  distclean                      realclean     
+  distdir                        skipcheck     
+  distmeta                       test          
+  distsign                       testdb        
   disttest                       versioninstall
+  docs
 
 You can run the 'help' action for a complete list of actions.
 
@@ -190,7 +199,8 @@ C<ACTION_whatever>, and then you could perform the action C<Build
 whatever>.
 
 For information on providing backward compatibility with
-C<ExtUtils::MakeMaker>, see L<Module::Build::Compat>.
+C<ExtUtils::MakeMaker>, see L<Module::Build::Compat> and
+L<http://www.makemaker.org/wiki/index.cgi?ModuleBuildConversionGuide>.
 
 =head1 METHODS
 
@@ -497,11 +507,31 @@ For backward compatibility, you may use the parameter C<scripts>
 instead of C<script_files>.  Please consider this usage deprecated,
 though it will continue to exist for several version releases.
 
+=item test_files
+
+An optional parameter specifying a set of files that should be used as
+C<Test::Harness>-style regression tests to be run during the C<test>
+action.  May be given as an array reference of the files, or as a hash
+reference whose keys are the files (and whose values will currently be
+ignored).  If the argument is given as a single string (not in an
+array reference), that string will be treated as a C<glob()> pattern
+specifying the files to use.
+
+The default is to look for a F<test.pl> script in the top-level
+directory of the distribution, and any files matching the glob pattern
+C<*.t> in the F<t/> subdirectory.  If the C<recursive_test_files>
+property is true, then the C<t/> directory will be scanned recursively
+for C<*.t> files.
+
 =item autosplit
 
 An optional C<autosplit> argument specifies a file which should be run
-through the C<Autosplit::autosplit()> function.  In general I don't
-consider this a great idea, because it's not always clear that
+through the C<Autosplit::autosplit()> function.  If multiple files
+should be split, the argument may be given as an array of the files to
+split.
+
+In general I don't consider autosplitting a great idea, because it's
+not always clear that
 autosplitting achieves its intended performance benefits.  It may even
 harm performance in environments like mod_perl, where as much as
 possible of a module's code should be loaded during startup.
@@ -559,28 +589,38 @@ To link your XS code against glib you might write something like:
      extra_linker_flags   => scalar `glib-config --libs`,
  );
 
+=item include_dirs
+
+Specifies any additional directories in which to search for C header
+files.  May be given as a string indicating a single directory, or as
+a list reference indicating multiple directories.
+
+
 =item dist_author
 
-This should be something like "John Doe <jdoe@example.com>".  This is
-used when creating PPD files.  If this is not specified, then
-C<Module::Build> looks at the module from which it gets the
-distribution's version.  If it finds a POD section marked "=head1
-AUTHOR", then it uses the contents of this section.
+This should be something like "John Doe <jdoe@example.com>", or if
+there are multiple authors, an anonymous array of strings may be
+specified.  This is used when generating metadata for F<META.yml> and
+PPD files.  If this is not specified, then C<Module::Build> looks at
+the module from which it gets the distribution's version.  If it finds
+a POD section marked "=head1 AUTHOR", then it uses the contents of
+this section.
 
 =item dist_abstract
 
 This should be a short description of the distribution.  This is used
-when creating PPD files.  If it is not given then C<Module::Build>
-looks in the POD of the module from which it gets the distribution's
-version.  It looks for the first line matching C<$package\s-\s(.+)>,
-and uses the captured text as the abstract.
+when generating metadata for F<META.yml> and PPD files.  If it is not
+given then C<Module::Build> looks in the POD of the module from which
+it gets the distribution's version.  It looks for the first line
+matching C<$package\s-\s(.+)>, and uses the captured text as the
+abstract.
 
 =back
 
 =item subclass()
 
 This creates a new C<Module::Build> subclass on the fly, as described
-in the L<SUBCLASSING> section.  The caller must provide either a
+in the L<"SUBCLASSING"> section.  The caller must provide either a
 C<class> or C<code> parameter, or both.  The C<class> parameter
 indicates the name to use for the new subclass, and defaults to
 C<MyModuleBuilder>.  The C<code> parameter specifies Perl code to use
@@ -671,7 +711,7 @@ It parses the command-line arguments into an action and an argument
 list, then calls the appropriate routine to handle the action.
 Currently (though this may change), an action C<foo> will invoke the
 C<ACTION_foo> method.  All arguments (including everything mentioned
-in L<ACTIONS> below) are contained in the C<< $self->{args} >> hash
+in L<"ACTIONS"> below) are contained in the C<< $self->{args} >> hash
 reference.
 
 =item os_type()
@@ -837,8 +877,8 @@ This method may be called as a class or object method.
 
 =item script_files()
 
-Returns an array reference specifying the perl script files to be
-installed.  This corresponds to the C<script_files> parameter to the
+Returns a hash reference whose keys are the perl script files to be
+installed, if any.  This corresponds to the C<script_files> parameter to the
 C<new()> method.  With an optional argument, this parameter may be set
 dynamically.
 
@@ -1158,10 +1198,27 @@ what the C<manifest> action would do, without actually doing anything.
 =item dist
 
 This action is helpful for module authors who want to package up their
-module for distribution through a medium like CPAN.  It will create a
+module for source distribution through a medium like CPAN.  It will create a
 tarball of the files listed in F<MANIFEST> and compress the tarball using
 GZIP compression.
 
+By default, this action will use the external C<tar> and C<gzip>
+executables on Unix-like platforms, and the C<Archive::Tar> module
+elsewhere.  However, you can force it to use whatever executable you
+want by supplying an explicit C<tar> (and optional C<gzip>) parameter:
+
+ perl Build dist --tar C:\path\to\tar.exe --gzip C:\path\to\zip.exe
+
+=item ppmdist
+
+Generates a PPM binary distribution and a PPD description file.  This
+action also invokes the 'ppd' action, so it can accept the same
+C<codebase> argument described under that action.
+
+This uses the same mechanism as the C<dist> action to tar & zip its
+output, so you can supply C<tar> and/or C<gzip> parameters to affect
+the result.
+
 =item distsign
 
 Uses C<Module::Signature> to create a SIGNATURE file for your
@@ -1456,6 +1513,7 @@ creating a separate file for your module:
   ------ in Build.PL: ----------
   #!/usr/bin/perl
   
+  use Module::Build;
   my $class = Module::Build->subclass
     (
      class => 'My::Builder',
@@ -1567,13 +1625,15 @@ signature or the like, if available.  See C<cons> for an example.
 
 - append to perllocal.pod
 - write .packlist in appropriate location (needed for un-install)
+- add a 'plugin' functionality
 
 =head1 AUTHOR
 
-Ken Williams, ken@mathforum.org
+Ken Williams, kwilliams@cpan.org
 
 Development questions, bug reports, and patches should be sent to the
 Module-Build mailing list at module-build-general@lists.sourceforge.net .
+
 Bug reports are also welcome at
 http://rt.cpan.org/NoAuth/Bugs.html?Dist=Module-Build .
 
@@ -1583,7 +1643,7 @@ details of how to access it.
 
 =head1 SEE ALSO
 
-perl(1), ExtUtils::MakeMaker(3), YAML(3)
+perl(1), Module::Build::Cookbook(3), ExtUtils::MakeMaker(3), YAML(3)
 
 http://www.dsmit.com/cons/
 
