@@ -35,10 +35,11 @@ sub resume {
   
   $self->read_config;
   
-  my $perl = $self->find_perl_interpreter;
-  warn(" * WARNING: Configuration was initially created with '$self->{properties}{perl}',\n".
-       "   but we are now using '$perl'.\n")
-    unless $perl eq $self->{properties}{perl};
+  unless ($self->_perl_is_same($self->{properties}{perl})) {
+    my $perl = $self->find_perl_interpreter;
+    warn(" * WARNING: Configuration was initially created with '$self->{properties}{perl}',\n".
+	 "   but we are now using '$perl'.\n");
+  }
   
   my $mb_version = Module::Build->VERSION;
   die(" * ERROR: Configuration was initially created with Module::Build version '$self->{properties}{mb_version}',\n".
@@ -47,6 +48,7 @@ sub resume {
   
   $self->cull_args(@ARGV);
   $self->{action} ||= 'build';
+  $self->_set_install_paths;
   
   return $self;
 }
@@ -58,7 +60,9 @@ sub new_from_context {
   
   # Run the Build.PL
   $package->run_perl_script('Build.PL');
-  return $package->resume;
+  my $self = $package->resume;
+  $self->merge_args(%args);
+  return $self;
 }
 
 sub current {
@@ -73,14 +77,6 @@ sub _construct {
   my $args   = delete $input{args}   || {};
   my $config = delete $input{config} || {};
 
-  # The following warning could be unnecessary if the user is running
-  # an embedded perl, but there aren't too many of those around, and
-  # embedded perls aren't usually used to install modules, and the
-  # installation process sometimes needs to run external scripts
-  # (e.g. to run tests).
-  my $perl = $package->find_perl_interpreter
-    or warn "Warning: Can't locate your perl binary";
-
   my $self = bless {
 		    args => {%$args},
 		    config => {%Config, %$config},
@@ -93,17 +89,25 @@ sub _construct {
 				   recommends      => {},
 				   build_requires  => {},
 				   conflicts       => {},
-				   perl            => $perl,
 				   mb_version      => Module::Build->VERSION,
 				   build_elements  => [qw( PL support pm xs pod script )],
-				   install_types   => [qw( lib arch script bindoc libdoc )],
 				   installdirs     => 'site',
+				   install_path    => {},
 				   include_dirs    => [],
 				   %input,
 				  },
 		   }, $package;
 
   my ($p, $c) = ($self->{properties}, $self->{config});
+
+  # The following warning could be unnecessary if the user is running
+  # an embedded perl, but there aren't too many of those around, and
+  # embedded perls aren't usually used to install modules, and the
+  # installation process sometimes needs to run external scripts
+  # (e.g. to run tests).
+  $p->{perl} = $self->find_perl_interpreter
+    or warn "Warning: Can't locate your perl binary";
+
   $p->{bindoc_dirs} ||= [ "$p->{blib}/script" ];
   $p->{libdoc_dirs} ||= [ "$p->{blib}/lib", "$p->{blib}/arch" ];
 
@@ -113,12 +117,6 @@ sub _construct {
 
   $self->add_to_cleanup( @{delete $p->{add_to_cleanup}} )
     if $p->{add_to_cleanup};
-
-  # perl 5.8.1-RC[1-3] had some broken %Config entries, and
-  # unfortunately Red Hat 9 shipped it like that.  Fix 'em up here.
-  for (qw(siteman1 siteman3 vendorman1 vendorman3)) {
-    $c->{"install${_}dir"} ||= $c->{"install${_}"};
-  }
   
   return $self;
 }
@@ -129,6 +127,8 @@ sub _set_install_paths {
   my $self = shift;
   my $c = $self->{config};
 
+  my @html = $c->{installhtmldir} ? (html => $c->{installhtmldir}) : ();
+
   $self->{properties}{install_sets} =
     {
      core   => {
@@ -138,6 +138,7 @@ sub _set_install_paths {
 		script  => $c->{installscript},
 		bindoc  => $c->{installman1dir},
 		libdoc  => $c->{installman3dir},
+		@html,
 	       },
      site   => {
 		lib     => $c->{installsitelib},
@@ -146,6 +147,7 @@ sub _set_install_paths {
 		script  => $c->{installsitescript} || $c->{installsitebin} || $c->{installscript},
 		bindoc  => $c->{installsiteman1dir} || $c->{installman1dir},
 		libdoc  => $c->{installsiteman3dir} || $c->{installman3dir},
+		@html,
 	       },
      vendor => {
 		lib     => $c->{installvendorlib},
@@ -154,6 +156,7 @@ sub _set_install_paths {
 		script  => $c->{installvendorscript} || $c->{installvendorbin} || $c->{installscript},
 		bindoc  => $c->{installvendorman1dir} || $c->{installman1dir},
 		libdoc  => $c->{installvendorman3dir} || $c->{installman3dir},
+		@html,
 	       },
     };
 }
@@ -163,12 +166,32 @@ sub cwd {
   return Cwd::cwd();
 }
 
+sub _perl_is_same {
+  my ($self, $perl) = @_;
+  return `$perl -MConfig=myconfig -e print -e myconfig` eq Config->myconfig;
+}
+
 sub find_perl_interpreter {
-  my $perl;
-  File::Spec->file_name_is_absolute($perl = $^X)
-    or -f ($perl = $Config::Config{perlpath})
-    or ($perl = $^X);
-  return $perl;
+  return $^X if File::Spec->file_name_is_absolute($^X);
+  my $proto = shift;
+  my $c = ref($proto) ? $proto->{config} : \%Config::Config;
+  my $exe = $c->{exe_ext};
+
+  my $thisperl = $^X;
+  if ($proto->os_type eq 'VMS') {
+    # VMS might have a file version at the end
+    $thisperl .= $exe unless $thisperl =~ m/$exe(;\d+)?$/i;
+  } elsif (defined $exe) {
+    $thisperl .= $exe unless $thisperl =~ m/$exe$/i;
+  }
+  
+  my @candidates = map File::Spec->catfile($_, $thisperl), File::Spec->path();
+  push @candidates, $c->{perlpath};
+  
+  foreach my $perl (@candidates) {
+    return $perl if -f $perl and $proto->_perl_is_same($perl);
+  }
+  return;
 }
 
 sub base_dir { shift()->{properties}{base_dir} }
@@ -218,15 +241,51 @@ sub y_n {
   }
 }
 
-sub notes {
+sub _general_notes {
   my $self = shift;
-  return $self->_persistent_hash_read('notes') unless @_;
+  my $type = shift;
+  return $self->_persistent_hash_read($type) unless @_;
   
   my $key = shift;
-  return $self->_persistent_hash_read('notes', $key) unless @_;
+  return $self->_persistent_hash_read($type, $key) unless @_;
   
   my $value = shift;
-  return $self->_persistent_hash_write('notes', { $key => $value });
+  $self->has_build_config(1) if $type =~ /^(build_config|features)$/;
+  return $self->_persistent_hash_write($type, { $key => $value });
+}
+
+sub notes        { shift()->_general_notes('notes', @_) }
+sub build_config { shift()->_general_notes('build_config', @_) }
+sub features     { shift()->_general_notes('features', @_) }
+
+sub ACTION_build_config {
+  my $self = shift;
+  return unless $self->has_build_config;
+  
+  die "The build_config feature requires that 'module_name' be set" unless $self->module_name;
+  my $notes_name = $self->module_name . '::BuildConfig';
+  my $notes_pm = File::Spec->catfile($self->blib, 'lib', split /::/, "$notes_name.pm");
+
+  print "Writing config notes to $notes_pm\n";
+  
+  my $fh = IO::File->new("> $notes_pm") or die "Can't create '$notes_pm': $!";
+
+  printf $fh <<'EOF', $notes_name;
+package %s;
+use strict;
+my $arrayref = eval do {local $/; <DATA>}
+  or die "Couldn't load BuildConfig data: $@";
+close DATA;
+my ($notes, $features) = @$arrayref;
+
+sub get { $notes->{$_[1]} }
+sub feature { $features->{$_[1]} }
+
+EOF
+  print $fh "__DATA__\n";
+
+  local $Data::Dumper::Terse = 1;
+  print $fh Data::Dumper::Dumper([scalar $self->build_config, scalar $self->features]);
 }
 
 {
@@ -249,12 +308,13 @@ sub notes {
        scripts
        script_files
        test_files
+       recursive_test_files
        perl
        config_dir
        blib
+       has_build_config
        build_script
        build_elements
-       install_types
        install_sets
        install_path
        install_base
@@ -267,6 +327,7 @@ sub notes {
        create_makefile_pl
        create_readme
        pollute
+       extra_compiler_flags
        include_dirs
        bindoc_dirs
        libdoc_dirs
@@ -286,6 +347,13 @@ sub notes {
   }
 }
 
+sub extra_compiler_flags {
+  my $self = shift;
+  my $p = $self->{properties};
+  $p->{extra_compiler_flags} = [@_] if @_;
+  return ref($p->{extra_compiler_flags}) ? $p->{extra_compiler_flags} : [$p->{extra_compiler_flags}];
+}
+
 # XXX Problem - if Module::Build is loaded from a different directory,
 # it'll look for (and perhaps destroy/create) a _build directory.
 sub subclass {
@@ -744,7 +812,7 @@ $quoted_INC
 use $build_package;
 
 if (-e 'Build.PL' and not $build_package->up_to_date("Build.PL", \$0)) {
-   warn "Warning: Build.PL has been altered.  You may need to run 'perl Build.PL' again.\n";
+   warn "Warning: Build.PL has been altered.  You may need to run 'perl Build.PL' again.\\n";
 }
 
 # This should have just enough arguments to be able to bootstrap the rest.
@@ -825,7 +893,7 @@ sub _call_action {
   return $self->$method();
 }
 
-sub _cull_arg {
+sub _read_arg {
   my ($self, $args, $key, $val) = @_;
 
   if ( exists $args->{$key} ) {
@@ -836,15 +904,15 @@ sub _cull_arg {
   }
 }
 
-sub cull_args {
+sub read_args {
   my $self = shift;
   my ($action, %args, @argv);
   while (@_) {
     local $_ = shift;
     if ( /^(\w+)=(.*)/ ) {
-      $self->_cull_arg(\%args, $1, $2);
+      $self->_read_arg(\%args, $1, $2);
     } elsif ( /^--(\w+)$/ ) {
-      $self->_cull_arg(\%args, $1, shift());
+      $self->_read_arg(\%args, $1, shift());
     } elsif ( /^(\w+)$/ and !defined($action)) {
       $action = $1;
     } else {
@@ -854,8 +922,7 @@ sub cull_args {
   $args{ARGV} = \@argv;
 
   # 'config' and 'install_path' are additive by hash key
-  my %additive = (config => $self->{config},
-		  install_path => $self->{properties}{install_path} ||= {});
+  my %additive = map {$_, 1} qw(config install_path);
 
   # Hashify these parameters
   for (keys %additive) {
@@ -870,8 +937,20 @@ sub cull_args {
     }
     $args{$_} = \%hash;
   }
+  
+  if ($args{makefile_env_macros}) {
+    require Module::Build::Compat;
+    %args = (%args, Module::Build::Compat->makefile_to_build_macros);
+  }
+  
+  return \%args, $action;
+}
+
+sub merge_args {
+  my ($self, $action, %args) = @_;
+  my %additive = (config => $self->{config},
+		  install_path => $self->{properties}{install_path});
 
-  # Now merge data into $self.
   $self->{action} = $action if defined $action;
 
   # Extract our 'properties' from $cmd_args, the rest are put in 'args'.
@@ -886,7 +965,12 @@ sub cull_args {
       $add_to->{$key} = $val;
     }
   }
+}
 
+sub cull_args {
+  my $self = shift;
+  my ($args, $action) = $self->read_args(@_);
+  $self->merge_args($action, %$args);
 }
 
 sub super_classes {
@@ -914,7 +998,7 @@ sub known_actions {
   return wantarray ? sort keys %actions : \%actions;
 }
 
-sub _get_action_docs {
+sub get_action_docs {
   my ($self, $action, $actions) = @_;
   $actions ||= $self->known_actions;
   $@ = '';
@@ -939,9 +1023,9 @@ sub _get_action_docs {
     # Look for our action
     my ($found, $inlist) = (0, 0);
     while (<$fh>) {
-      if (/^=item\s+\Q$action\E\b/o)  {
+      if (/^=item\s+\Q$action\E\b/)  {
 	$found = 1;
-      } elsif (/^=item/) {
+      } elsif (/^=(item|back)/) {
 	last if $found > 1 and not $inlist;
       }
       next unless $found;
@@ -951,12 +1035,17 @@ sub _get_action_docs {
       ++$found  if /^\w/; # Found descriptive text
     }
   }
-  ($@ = "Sorry, couldn't find any documentation to search.\n"), return
-    unless $files_found;
-  ($@ = "Couldn't find any docs for action '$action'.\n"), return
-    unless @docs;
+
+  unless ($files_found) {
+    $@ = "Couldn't find any documentation to search";
+    return;
+  }
+  unless (@docs) {
+    $@ = "Couldn't find any docs for action '$action'";
+    return;
+  }
   
-  return @docs;
+  return join '', @docs;
 }
 
 sub ACTION_help {
@@ -964,7 +1053,8 @@ sub ACTION_help {
   my $actions = $self->known_actions;
   
   if (@{$self->{args}{ARGV}}) {
-    print $self->_get_action_docs($self->{args}{ARGV}[0], $actions), $@;
+    my $msg = $self->get_action_docs($self->{args}{ARGV}[0], $actions) || "$@\n";
+    print $msg;
     return;
   }
 
@@ -1015,7 +1105,7 @@ sub ACTION_test {
 		File::Spec->catdir($p->{base_dir}, $self->blib, 'arch'),
 		@INC);
   
-  my $tests = $self->test_files;
+  my $tests = $self->find_test_files;
 
   if (@$tests) {
     # Work around a Test::Harness bug that loses the particular perl we're running under
@@ -1028,7 +1118,7 @@ sub ACTION_test {
   # This will get run and the user will see the output.  It doesn't
   # emit Test::Harness-style output.
   if (-e 'visual.pl') {
-    $self->run_perl_script('visual.pl', '-Mblib');
+    $self->run_perl_script('visual.pl', '-Mblib='.$self->blib);
   }
 }
 
@@ -1036,25 +1126,14 @@ sub test_files {
   my $self = shift;
   my $p = $self->{properties};
   if (@_) {
-    $p->{test_files} = (@_ == 1 ? shift : [@_]);
+    return $p->{test_files} = (@_ == 1 ? shift : [@_]);
   }
-
-  my @tests;
-  if ($p->{test_files}) {
-    @tests = (map { -d $_ ? $self->expand_test_dir($_) : $_ }
-	      map glob,
-	      $self->split_like_shell($p->{test_files}));
-  } else {
-    # Find all possible tests in t/ or test.pl
-    push @tests, 'test.pl'                          if -e 'test.pl';
-    push @tests, $self->expand_test_dir('t')        if -e 't' and -d _;
-  }
-  return \@tests;
+  return $self->find_test_files;
 }
 
 sub expand_test_dir {
   my ($self, $dir) = @_;
-  return @{$self->rscan_dir($dir, qr{\.t$})} if $self->{properties}{recursive_test_files};
+  return sort @{$self->rscan_dir($dir, qr{\.t$})} if $self->recursive_test_files;
   return sort glob File::Spec->catfile($dir, "*.t");
 }
 
@@ -1073,7 +1152,9 @@ sub ACTION_code {
   $self->add_to_cleanup($blib);
   File::Path::mkpath( File::Spec->catdir($blib, 'arch') );
   
-  $self->autosplit_file($self->autosplit, $blib) if $self->autosplit;
+  if (my $split = $self->autosplit) {
+    $self->autosplit_file($_, $blib) for ref($split) ? @$split : ($split);
+  }
   
   foreach my $element (@{$self->build_elements}) {
     my $method = "process_${element}_files";
@@ -1085,6 +1166,7 @@ sub ACTION_build {
   my $self = shift;
   $self->depends_on('code');
   $self->depends_on('docs');
+  $self->depends_on('build_config');
 }
 
 sub process_support_files {
@@ -1105,6 +1187,7 @@ sub process_PL_files {
   my $files = $self->find_PL_files;
   
   while (my ($file, $to) = each %$files) {
+    local $ENV{'PERL5LIB'} = join $self->{config}{path_sep}, @INC;
     unless ($self->up_to_date( $file, $to )) {
       $self->run_perl_script($file, [], [@$to]);
       $self->add_to_cleanup(@$to);
@@ -1188,9 +1271,7 @@ sub find_xs_files  { shift->_find_file_by_type('xs',  'lib') }
 
 sub find_script_files {
   my $self = shift;
-  if (my $files = $self->{properties}{"script_files"}) {
-    $files = { map {$_, undef} @$files } if UNIVERSAL::isa($files, 'ARRAY');
-    
+  if (my $files = $self->script_files) {
     # Always given as a Unix file spec.  Values in the hash are
     # meaningless, but we preserve if present.
     return { map {$self->localize_file_path($_), $files->{$_}} keys %$files };
@@ -1200,6 +1281,29 @@ sub find_script_files {
   return {};
 }
 
+sub find_test_files {
+  my $self = shift;
+  my $p = $self->{properties};
+  
+  if (my $files = $p->{test_files}) {
+    $files = [keys %$files] if UNIVERSAL::isa($files, 'HASH');
+    $files = [map { -d $_ ? $self->expand_test_dir($_) : $_ }
+	      map glob,
+	      $self->split_like_shell($files)];
+    
+    # Always given as a Unix file spec.  Values in the hash are
+    # meaningless, but we preserve if present.
+    return [ map $self->localize_file_path($_), @$files ];
+    
+  } else {
+    # Find all possible tests in t/ or test.pl
+    my @tests;
+    push @tests, 'test.pl'                          if -e 'test.pl';
+    push @tests, $self->expand_test_dir('t')        if -e 't' and -d _;
+    return \@tests;
+  }
+}
+
 sub _find_file_by_type {
   my ($self, $type, $dir) = @_;
   
@@ -1232,6 +1336,7 @@ sub fix_shebang_line { # Adapted from fixin() in ExtUtils::MM_Unix 1.35
     next unless $line =~ s/^\s*\#!\s*//;     # Not a shbang file.
     
     my ($cmd, $arg) = (split(' ', $line, 2), '');
+    next unless $cmd =~ /perl/i;
     my $interpreter = $self->{properties}{perl};
     
     print STDOUT "Changing sharpbang in $file to $interpreter" if $self->{verbose};
@@ -1275,6 +1380,7 @@ sub ACTION_docs {
   require Pod::Man;
   $self->manify_bin_pods() if $self->install_destination('bindoc');
   $self->manify_lib_pods() if $self->install_destination('libdoc');
+  $self->htmlify_pods()    if $self->install_destination('html');
 }
 
 sub manify_bin_pods {
@@ -1339,6 +1445,95 @@ sub contains_pod {
   return '';
 }
 
+sub ACTION_html {
+  my $self = shift;
+  $self->depends_on('code');
+  $self->htmlify_pods;
+}
+
+sub htmlify_pods {
+  my $self = shift;
+  require Pod::Html;
+  require Module::Build::PodParser;
+  
+  my $blib = $self->blib;
+  my $html = File::Spec::Unix->catdir($blib, 'html');
+  my $script = File::Spec::Unix->catdir($blib, 'script');
+  
+  unless (-d $html) {
+    File::Path::mkpath($html, 1, 0755) or die "Couldn't mkdir $html: $!";
+  }
+  
+  my $pods = $self->_find_pods([$self->blib]);
+  if (-d $script) {
+    File::Find::finddepth( sub 
+                           {$pods->{$File::Find::name} = 
+                              "script::" . File::Basename::basename($File::Find::name) 
+                                if (-f $_ and not /\.bat$/ and $self->contains_pod($_));
+                          }, $script);
+  }
+  
+  foreach my $pod (keys %$pods){
+    $self->_htmlify_pod(
+			path => $pod,
+			rel_path => $pods->{$pod},
+			htmldir => $html,
+			backlink => '__top',
+			css => ($^O =~ /Win32/) ? 'Active.css' : '',
+		       );
+  }
+}
+
+sub _htmlify_pod {
+  my ($self, %args) = @_;
+  
+  my ($name, $path) = File::Basename::fileparse($args{rel_path}, qr{\..*});
+  my @dirs = File::Spec->splitdir($path);
+  my $isbin = shift @dirs eq 'script';
+  my $infile = File::Spec::Unix->abs2rel($args{path});
+    
+  my @rootdirs  = $isbin? ('bin') : ('site', 'lib');
+  
+  my $fulldir = File::Spec::Unix->catfile($args{htmldir}, @rootdirs, @dirs);
+  my $outfile = File::Spec::Unix->catfile($fulldir, $name . '.html');
+
+  return if $self->up_to_date($infile, $outfile);
+    
+  unless (-d $fulldir){
+    File::Path::mkpath($fulldir, 1, 0755) 
+	or die "Couldn't mkdir $fulldir: $!";  
+  }
+    
+  my $path2root = "../" x (@rootdirs+@dirs-1);
+  my $htmlroot = File::Spec::Unix->catdir($path2root, 'site');
+  my $podpath = join ":" => ($isbin ? qw(bin lib) : qw(lib));
+  my $title = join('::', @dirs) . $name;
+    
+  {
+    my $fh = IO::File->new($infile);
+    my $abstract = Module::Build::PodParser->new(fh => $fh)->get_abstract();
+    $title .= " - $abstract" if $abstract;
+  }
+    
+  my $blib = $self->blib;
+  my @opts = (
+	      '--header',
+	      '--flush',
+	      "--backlink=$args{backlink}",
+	      "--title=$title",
+	      "--podpath=$podpath",
+	      "--infile=$infile",
+	      "--outfile=$outfile",
+	      "--podroot=$blib",
+	      "--htmlroot=$htmlroot",
+	     );
+  push @opts, "--css=$path2root/$args{css}" if $args{css};
+    
+  print "Creating $outfile\n";
+  print "pod2html @opts\n" if $self->verbose;
+  Pod::Html::pod2html(@opts);	# or warn "pod2html @opts failed: $!";
+}
+
 # Adapted from ExtUtils::MM_Unix
 sub man1page_name {
   my $self = shift;
@@ -1380,11 +1575,11 @@ sub ACTION_diff {
   my $text_suffix = qr{\.(pm|pod)$};
 
   while (my $localdir = each %$installmap) {
+    my @localparts = File::Spec->splitdir($localdir);
     my $files = $self->rscan_dir($localdir, sub {-f});
     
     foreach my $file (@$files) {
       my @parts = File::Spec->splitdir($file);
-      my @localparts = File::Spec->splitdir($localdir);
       @parts = @parts[@localparts .. $#parts]; # Get rid of blib/lib or similar
       
       my $installed = $self->find_module_by_name(join('::', @parts), \@myINC);
@@ -1449,11 +1644,19 @@ sub ACTION_realclean {
 sub ACTION_ppd {
   my ($self) = @_;
   require Module::Build::PPMMaker;
-  my $ppd = Module::Build::PPMMaker->new(archname => $self->{config}{archname});
+  my $ppd = Module::Build::PPMMaker->new();
   my $file = $ppd->make_ppd(%{$self->{args}}, build => $self);
   $self->add_to_cleanup($file);
 }
 
+sub ACTION_ppmdist {
+  my ($self) = @_;
+  
+  $self->depends_on('build', 'ppd');
+  $self->add_to_cleanup($self->ppm_name);
+  $self->make_tarball($self->blib, $self->ppm_name);
+}
+
 sub ACTION_dist {
   my ($self) = @_;
   
@@ -1530,29 +1733,34 @@ sub ACTION_distclean {
   $self->depends_on('distcheck');
 }
 
+sub do_create_makefile_pl {
+  my $self = shift;
+  require Module::Build::Compat;
+  Module::Build::Compat->create_makefile_pl($self->create_makefile_pl, $self, @_);
+}
+
+sub do_create_readme {
+  my $self = shift;
+  require Pod::Text;
+  my $parser = Pod::Text->new;
+  $parser->parse_from_file($self->dist_version_from, 'README', @_);
+}
+
 sub ACTION_distdir {
   my ($self) = @_;
 
   $self->depends_on('distmeta');
 
-  if ($self->create_makefile_pl) {
-    require Module::Build::Compat;
-    Module::Build::Compat->create_makefile_pl($self->create_makefile_pl, $self);
-  }
-  
-  if ($self->create_readme) {
-    require Pod::Text;
-    my $parser = Pod::Text->new;
-    $parser->parse_from_file($self->dist_version_from, 'README');
-  }
+  $self->do_create_makefile_pl if $self->create_makefile_pl;
+  $self->do_create_readme if $self->create_readme;
   
   my $dist_files = $self->_read_manifest('MANIFEST');
   delete $dist_files->{SIGNATURE};  # Don't copy, create a fresh one
-  unless (keys %$dist_files) {
-    warn "No files found in MANIFEST - try running 'manifest' action?\n";
-    return;
-  }
-  warn "*** Did you forget to add $self->{metafile} to the MANIFEST?\n" unless exists $dist_files->{$self->{metafile}};
+  die "No files found in MANIFEST - try running 'manifest' action?\n"
+    unless ($dist_files and keys %$dist_files);
+  
+  warn "*** Did you forget to add $self->{metafile} to the MANIFEST?\n"
+    unless exists $dist_files->{$self->{metafile}};
   
   my $dist_dir = $self->dist_dir;
   $self->delete_filetree($dist_dir);
@@ -1582,9 +1790,56 @@ sub ACTION_disttest {
   chdir $start_dir;
 }
 
+sub _write_default_maniskip {
+  my $self = shift;
+  my $file = shift || 'MANIFEST.SKIP';
+  my $fh = IO::File->new("> $file")
+    or die "Can't open $file: $!";
+
+  # This is pretty much straight out of
+  # MakeMakers default MANIFEST.SKIP file
+  print $fh <<'EOF';
+# Avoid version control files.
+\bRCS\b
+\bCVS\b
+,v$
+\B\.svn\b
+
+# Avoid Makemaker generated and utility files.
+\bMakefile$
+\bblib
+\bMakeMaker-\d
+\bpm_to_blib$
+\bblibdirs$
+^MANIFEST\.SKIP$
+
+# Avoid Module::Build generated and utility files.
+\bBuild$
+\b_build
+
+# Avoid temp and backup files.
+~$
+\.old$
+\.bak$
+\#$
+\b\.#
+EOF
+
+  $fh->close();
+}
+
 sub ACTION_manifest {
   my ($self) = @_;
-  
+
+  my $metafile = $self->{metafile} || 'META.yml';
+  $self->depends_on('distmeta') unless -e $metafile;
+
+  my $maniskip = 'MANIFEST.SKIP';
+  unless ( -e 'MANIFEST' || -e $maniskip ) {
+    warn "File '$maniskip' does not exist: Creating a default '$maniskip'\n";
+    $self->_write_default_maniskip($maniskip);
+  }
+
   require ExtUtils::Manifest;  # ExtUtils::Manifest is not warnings clean.
   local ($^W, $ExtUtils::Manifest::Quiet) = (0,1);
   ExtUtils::Manifest::mkmanifest();
@@ -1595,12 +1850,23 @@ sub dist_dir {
   return "$self->{properties}{dist_name}-$self->{properties}{dist_version}";
 }
 
+sub ppm_name {
+  my $self = shift;
+  return 'PPM-' . $self->dist_dir;
+}
+
 sub script_files {
   my $self = shift;
-  if (@_) {
-    $self->{properties}{script_files} = ref($_[0]) ? $_[0] : [@_];
+  
+  for ($self->{properties}{script_files}) {
+    $_ = shift if @_;
+    return unless $_;
+    
+    # Always coerce into a hash
+    return $_ if UNIVERSAL::isa($_, 'HASH');
+    return $_ = {$_ => 1} unless ref();
+    return { map {$_,1} @$_ };
   }
-  return $self->{properties}{script_files};
 }
 BEGIN { *scripts = \&script_files; }
 
@@ -1608,6 +1874,28 @@ sub valid_licenses {
   return { map {$_, 1} qw(perl gpl artistic lgpl bsd open_source unrestricted restrictive unknown) };
 }
 
+sub _write_minimal_metadata {
+  my $self = shift;
+  my $p = $self->{properties};
+
+  my $file = $self->{metafile};
+  my $fh = IO::File->new("> $file")
+    or die "Can't open $file: $!";
+
+  print $fh <<"END_OF_META";
+--- #YAML:1.0
+name: $p->{dist_name}
+version: $p->{dist_version}
+author:
+@{[ join "\n", map "  - $_", @{$self->dist_author} ]}
+abstract: @{[ $self->dist_abstract ]}
+license: $p->{license}
+generated_by: Module::Build version @{[ Module::Build->VERSION ]}, without YAML.pm
+END_OF_META
+
+  $fh->close();
+}
+
 sub ACTION_distmeta {
   my ($self) = @_;
   return if $self->{wrote_metadata};
@@ -1623,30 +1911,35 @@ sub ACTION_distmeta {
     die "Unknown license type '$p->{license}";
   }
 
+  # If we're in the distdir, the metafile may exist and be non-writable.
+  $self->delete_filetree($self->{metafile});
+
   unless (eval {require YAML; 1}) {
-    warn "Couldn't load YAML.pm: $@\n";
-    return;
+    warn <<EOM;
+\nCouldn't load YAML.pm, generating a minimal META.yml without it.
+Please check and edit the generated metadata, or consider installing YAML.pm.\n
+EOM
+
+    return $self->_write_minimal_metadata();
   }
 
   # We use YAML::Node to get the order nice in the YAML file.
   my $node = YAML::Node->new({});
   
-  $node->{name} = $p->{dist_name};
-  $node->{version} = $p->{dist_version};
-  $node->{license} = $p->{license};
-  $node->{distribution_type} = 'module';
+  foreach (qw(dist_name dist_version dist_author dist_abstract license)) {
+    (my $name = $_) =~ s/^dist_//;
+    $node->{$name} = $self->$_();
+  }
 
-  foreach (qw(requires recommends build_requires conflicts dynamic_config)) {
-    $node->{$_} = $p->{$_} if exists $p->{$_};
+  foreach (qw(requires recommends build_requires conflicts)) {
+    $node->{$_} = $p->{$_} if exists $p->{$_} and keys %{ $p->{$_} };
   }
   
+  $node->{dynamic_config} = $p->{dynamic_config} if exists $p->{dynamic_config};
   $node->{provides} = $self->find_dist_packages;
 
   $node->{generated_by} = "Module::Build version " . Module::Build->VERSION;
   
-  # If we're in the distdir, the metafile may exist and be non-writable.
-  $self->delete_filetree($self->{metafile});
-
   # YAML API changed after version 0.30
   my $yaml_sub = $YAML::VERSION le '0.30' ? \&YAML::StoreFile : \&YAML::DumpFile;
   return $self->{wrote_metadata} = $yaml_sub->($self->{metafile}, $node );
@@ -1654,9 +1947,10 @@ sub ACTION_distmeta {
 
 sub _read_manifest {
   my ($self, $file) = @_;
+  return undef unless -e $file;
+
   require ExtUtils::Manifest;  # ExtUtils::Manifest is not warnings clean.
   local ($^W, $ExtUtils::Manifest::Quiet) = (0,1);
-
   return scalar ExtUtils::Manifest::maniread($file);
 }
 
@@ -1697,13 +1991,23 @@ sub _packages_inside {
 }
 
 sub make_tarball {
-  my ($self, $dir) = @_;
+  my ($self, $dir, $file) = @_;
+  $file ||= $dir;
   
-  require Archive::Tar;
-  my $files = $self->rscan_dir($dir);
+  print "Creating $file.tar.gz\n";
   
-  print "Creating $dir.tar.gz\n";
-  Archive::Tar->create_archive("$dir.tar.gz", 1, @$files);
+  if ($self->{args}{tar}) {
+    my $tar_flags = $self->{properties}{verbose} ? 'cvf' : 'cf';
+    $self->do_system($self->{args}{tar}, $tar_flags, "$file.tar", $dir);
+    $self->do_system($self->{args}{gzip}, "$file.tar") if $self->{args}{gzip};
+  } else {
+    require Archive::Tar;
+    # Archive::Tar versions >= 1.09 use the following to enable a compatibility
+    # hack so that the resulting archive is compatible with older clients.
+    $Archive::Tar::DO_NOT_USE_PREFIX = 0;
+    my $files = $self->rscan_dir($dir);
+    Archive::Tar->create_archive("$file.tar.gz", 1, @$files);
+  }
 }
 
 sub install_base_relative {
@@ -1733,7 +2037,9 @@ sub install_destination {
 
 sub install_types {
   my $self = shift;
-  return @{ $self->{properties}{install_types} }
+  my $p = $self->{properties};
+  my %types = (%{$p->{install_path}}, %{ $p->{install_sets}{$p->{installdirs}} });
+  return sort keys %types;
 }
 
 sub install_map {
@@ -1745,8 +2051,13 @@ sub install_map {
     my $localdir = File::Spec->catdir( $blib, $type );
     next unless -e $localdir;
     
-    $map{$localdir} = $self->install_destination($type)
-      or die "Can't figure out where to install things of type '$type'";
+    if (my $dest = $self->install_destination($type)) {
+      $map{$localdir} = $dest;
+    } else {
+      # Platforms like Win32, MacOS, etc. may not build man pages
+      die "Can't figure out where to install things of type '$type'"
+	unless $type =~ /^(lib|bin)doc$/;
+    }
   }
 
   if (length(my $destdir = $self->{properties}{destdir} || '')) {
@@ -1853,7 +2164,7 @@ sub compile_c {
   my @cc = $self->split_like_shell($cf->{cc});
   
   $self->do_system(@cc, @flags, '-o', $obj_file, $file)
-    or die "error building $cf->{dlext} file from '$file'";
+    or die "error building $cf->{obj_ext} file from '$file'";
 
   return $obj_file;
 }
@@ -1890,11 +2201,12 @@ sub link_c {
   my ($self, $to, $file_base) = @_;
   my ($cf, $p) = ($self->{config}, $self->{properties}); # For convenience
 
+  my $obj_file = "$file_base$cf->{obj_ext}";
   my $lib_file = File::Spec->catfile($to, File::Basename::basename("$file_base.$cf->{dlext}"));
   $self->add_to_cleanup($lib_file);
   my $objects = $p->{objects} || [];
   
-  unless ($self->up_to_date(["$file_base$cf->{obj_ext}", @$objects], $lib_file)) {
+  unless ($self->up_to_date([$obj_file, @$objects], $lib_file)) {
     $self->prelink_c($to, $file_base) if $self->need_prelink_c;
 
     my @linker_flags = $self->split_like_shell($p->{extra_linker_flags});
@@ -1902,8 +2214,8 @@ sub link_c {
     my @shrp = $self->split_like_shell($cf->{shrpenv});
     my @ld = $self->split_like_shell($cf->{ld});
     $self->do_system(@shrp, @ld, @lddlflags, '-o', $lib_file,
-		     "$file_base$cf->{obj_ext}", @$objects, @linker_flags)
-      or die "error building $file_base$cf->{obj_ext} from '$file_base.$cf->{dlext}'";
+		     $obj_file, @$objects, @linker_flags)
+      or die "error building .$cf->{dlext} file from '$obj_file'";
   }
   
   return $lib_file;
